// Cocoa variant of the "minimal animation" demo originally written for Carbon
// Nibless application 
// By Ingemar 2009. ObjC version 2011.
// This is over 120 lines, QD/carbon version is 64 lines, CG/Carbon is 98. That's progress!

#import <Cocoa/Cocoa.h>

// -------------------- View ------------------------

@interface TestView : NSView { }
-(void)drawRect:(NSRect)rect;
@end

float loop;

void MyLine(float h1, float v1, float h2, float v2)
{
	NSPoint p1, p2;
	
	p1.x = h1;
	p1.y = v1;
	p2.x = h2;
	p2.y = v2;
	[NSBezierPath strokeLineFromPoint: p1 toPoint: p2];
};

#define Pi 3.1415

@implementation TestView

-(void)drawRect:(NSRect)rect
{
	NSRect cgr = NSMakeRect(100., 100., 200., 200.);
	NSBezierPath *p;
	NSColor *col;
	
	NSRect r;
	NSBezierPath *bp;
	NSColor *color;

	[[NSColor whiteColor] setFill];
	NSRectFill(rect);
	[[NSColor blueColor] setFill];
	
	loop = loop + 0.1;
	
	// Draw the spinning wheel
	
	p = [NSBezierPath bezierPathWithOvalInRect: cgr];
	col = [NSColor blackColor];
	[col set];
	[p stroke];

	MyLine(100, 100, 200 + (100*sin(loop)), 200 + (100*cos(loop)));
	MyLine(200 + (100*sin(loop)), 200 + (100*cos(loop)),
		200 + (100*sin(loop+Pi)), 200 + (100*cos(loop+Pi)));
	MyLine(200 + (100*sin(loop+Pi/2)), 200 + (100*cos(loop+Pi/2)),
		200 + (100*sin(loop-Pi/2)), 200 + (100*cos(loop-Pi/2)));
}

-(void)windowWillClose:(NSNotification *)note
{
    [[NSApplication sharedApplication] terminate:self];
}
@end


// -------------------- Timer ------------------------

// Mini-mini class for the timer
@interface TimerController : NSObject { }
-(void)timerFireMethod:(NSTimer *)t;
@end

NSTimer	*gTimer;
TimerController	*myTimerController;
NSView	*view;
	
// Timer!
@implementation TimerController
-(void)timerFireMethod:(NSTimer *)t;
{
	[view setNeedsDisplay: YES];
}

// ------------------ Main program ---------------------

NSApplication *myApp;
NSView *view;
NSAutoreleasePool *pool;
NSWindow *window;

int main(int argc, const char *argv[])
{
	pool = [NSAutoreleasePool new];
	myApp = [NSApplication sharedApplication];
	NSRect frame = NSMakeRect(10., 1000., 400., 400.);
	
	window = [NSWindow alloc];
	[window initWithContentRect:frame
                  styleMask:NSTitledWindowMask | NSClosableWindowMask | NSMiniaturizableWindowMask
                    backing:NSBackingStoreBuffered
                      defer:false];
    [window setTitle:@"Minimal animation (Cocoa)"];

	view = [TestView alloc];
	[view initWithFrame: frame];
	[window setContentView: view];
	[window setDelegate: view];
	[window makeKeyAndOrderFront: nil];

	// Timer
	myTimerController = [TimerController alloc];
	gTimer = [NSTimer
		scheduledTimerWithTimeInterval: 0.02
		target: myTimerController
		selector: @selector(timerFireMethod:)
		userInfo: nil
		repeats: YES];

	// Main loop
	[myApp run];
	[pool release]; // Free;
	return( EXIT_SUCCESS );
}
